import os, pygame
from pygame.locals import *

from helper import *
from monster import * # import all monsters
from item import * # import all items

class Level:
    def __init__(self):
        self.groundSurface = 0  # ground surface
        self.tileSurface = 0    # tile surface
        self.roomTitle = ""
        self.groundData = []    # ground data
        self.tileData = []      # tile data
        self.levelColumns = 0
        self.levelRows = 0

    def draw(self, screen, x,y):
        screen.blit(self.groundSurface, (-x,-y)) #this will be offset in the future
        screen.blit(self.tileSurface, (-x,-y))

    def tileAt(self, x, y):
        if x > self.levelColumns * 32 or y > self.levelRows * 32 or x < 0 or y < 0:
            #debug
            print 'Somebody just peeked outside the level at ' + str(x) + ', ' + str(y)
            return 'W' # not going to check outside the box
        if self.tileSurface.get_at((int(x),int(y))) == (0,255,0):
            return '0'
        else:
            column = int(x)/32
            row = int(y)/32
            line = self.tileData[row]
            return line[column]

    def loadLevel(self, level, monsterList, itemList):
        if level < 10:
            fullname = os.path.join('levels', 'floor0'+str(level)+'.txt')
        else:
            fullname = os.path.join('levels', 'floor'+str(level)+'.txt')
        openLevel = open(fullname, 'r')
        self.roomTitle = openLevel.readline().strip()
        line = openLevel.readline().strip()
        monsterData =   []
        itemData =      []
        if line == "Ground":
                #debug
                line = openLevel.readline().strip()
                while line != "Tiles":
                    self.groundData.append(line)
                    line = openLevel.readline().strip()
                line = openLevel.readline().strip()
                while line != "Items":
                    self.tileData.append(line)
                    line = openLevel.readline().strip()
                line = openLevel.readline().strip()
                while line != "Monsters":
                    itemData.append(line)
                    line = openLevel.readline().strip()
                line = openLevel.readline().strip()
                while line != "":       # python returns empty string at EOF
                    monsterData.append(line)
                    line = openLevel.readline().strip()
        # CLOSE IT UP
        openLevel.close()            
        # we have acquired level data, now lets put this to use
        self.levelColumns = len(self.groundData[0])
        self.levelRows = len(self.groundData)
        # make a ground surface
        self.groundSurface = pygame.Surface((self.levelColumns*32,self.levelRows*32))
        self.groundSurface.fill((125,125,125))
        # make a tile surface and fill it with transparancy
        self.tileSurface = pygame.Surface((self.levelColumns*32,self.levelRows*32))
        self.tileSurface.set_colorkey((0,255,0))
        self.tileSurface.fill((0,255,0))
        # setup tiles to blit
        gtile = []
        gtile0, null = load_image('groundTile0.gif')
        gtile1, null = load_image('groundTile1.gif')
        gtile2, null = load_image('groundTile2.gif')
        gtile3, null = load_image('groundTile3.gif')
        gtile4, null = load_image('groundTile4.gif')
        gtile.append(gtile0)
        gtile.append(gtile1)
        gtile.append(gtile2)
        gtile.append(gtile3)
        gtile.append(gtile4)
        ttile = []
        ttile0, null = load_image('wallTile1.gif')
        ttile1, null = load_image('redTile1.gif')
        ttile2, null = load_image('blueTile1.gif')
        ttile3, null = load_image('upTile1.gif',-1)
        ttile4, null = load_image('downTile1.gif')
        ttile.append(ttile0)
        ttile.append(ttile1)
        ttile.append(ttile2)
        ttile.append(ttile3)
        ttile.append(ttile4)

        playerStartX = 0
        playerStartY = 0
        for i in xrange(self.levelRows):
            curGround = self.groundData[i]
            curTiles = self.tileData[i]
            curItems = itemData[i]
            curMonsters = monsterData[i]
            for j in xrange(self.levelColumns):
                self.groundSurface.blit(gtile[int(curGround[j])],(j*32,i*32))
                if ( curTiles[j] != '0' ):
                    if curTiles[j] == 'W':
                        self.tileSurface.blit(ttile[0],(j*32,i*32))
                    elif curTiles[j] == 'R':
                        self.tileSurface.blit(ttile[1],(j*32,i*32))
                    elif curTiles[j] == 'B':
                        self.tileSurface.blit(ttile[2],(j*32,i*32))
                    elif curTiles[j] == 'U':
                        self.tileSurface.blit(ttile[3],(j*32,i*32))
                    elif curTiles[j] == 'D':
                        #self.tileSurface.blit(ttile[3],(j*32,i*32))
                        playerStartX = j*32
                        playerStartY = i*32+32
                if ( curItems[j] != '0' ):
                    if curItems[j] == 'R':
                        itemList.add(RedKey((j*32),(i*32)))
                    elif curItems[j] == 'B':
                        itemList.add(BlueKey((j*32),(i*32)))
                    elif curItems[j] == 'M':
                        itemList.add(Meat((j*32),(i*32)))
                if ( curMonsters[j] != '0' ):
                    if curMonsters[j] == 'L':
                        monsterList.add(LizardMan((j*32),(i*32)))
                    elif curMonsters[j] == 'S':
                        monsterList.add(SkullMan((j*32),(i*32)))
        # all done! return the max width & height of this level!
        return playerStartX, playerStartY, self.levelColumns*32, self.levelRows*32

    def removeDoor(self, x, y):
        """Looks at the x,y provided, if it is actually a red door instead of 0 or wall, we remove it"""
        blank,null = load_image('blank.gif')
        if self.tileData[int(y/32)][int(x/32)] == 'R':
            blitx = int(x/32)*32
            blity = int(y/32)*32
            self.tileSurface.blit(blank,(blitx,blity))
            self.tileData[(blity/32)] = str(self.tileData[(blity/32)][0:((blitx)/32)]) + '0' + str(self.tileData[(blity/32)][(((blitx)/32)+1):])
        elif self.tileData[int(y/32)][int(x/32)] == 'B':
            blitx = int(x/32)*32
            blity = int(y/32)*32
            self.tileSurface.blit(blank,(blitx,blity))
            self.tileData[(blity/32)] = str(self.tileData[(blity/32)][0:((blitx)/32)]) + '0' + str(self.tileData[(blity/32)][(((blitx)/32)+1):])
